import os
import gdal
import gdalnumeric
import osr
import numpy as np
from dcs4cop_vmm_postproc.helper import Helper
from dcs4cop_vmm_postproc import scaler


def merge2rgb(input_path,
              red_bandname, from_phys_red, to_phys_red,
              green_bandname, from_phys_green, to_phys_green,
              blue_bandname, from_phys_blue, to_phys_blue,
              output_nodata,
              output_file):

    input_raster = gdal.Open(input_path, gdal.GA_ReadOnly)
    input_crs = int((input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system']).split(':')[1])  # e.g. 32631
    input_raster = None

    red_array, red_band, red_raster = read_band(input_path, red_bandname)
    green_array, green_band, green_raster = read_band(input_path, green_bandname)
    blue_array, blue_band, blue_raster = read_band(input_path, blue_bandname)

#    red_array = rescale(red_array,
#                        int((from_phys_red - red_band.GetOffset()) / red_band.GetScale()),
#                        int((to_phys_red - red_band.GetOffset()) / red_band.GetScale()),
#                        red_band.GetNoDataValue(),
#                        0, output_nodata - 1, output_nodata)           
#    green_array = rescale(green_array,
#                          int((from_phys_green - green_band.GetOffset()) / green_band.GetScale()),
#                          int((to_phys_green - green_band.GetOffset()) / green_band.GetScale()),
#                          green_band.GetNoDataValue(),
#                          0, output_nodata - 1, output_nodata)
#    blue_array = rescale(blue_array,
#                         int((from_phys_blue - blue_band.GetOffset()) / blue_band.GetScale()),
#                         int((to_phys_blue - blue_band.GetOffset()) / blue_band.GetScale()),
#                         blue_band.GetNoDataValue(),
#                         0, output_nodata - 1, output_nodata)
#    
    red_array = scaler.scale_array(red_array, red_band.GetNoDataValue(), red_band.GetScale(), red_band.GetOffset(),
                                   to_phys_red, (to_phys_red/254))
                
    green_array = scaler.scale_array(green_array, green_band.GetNoDataValue(), green_band.GetScale(), green_band.GetOffset(),
                                     to_phys_green, (to_phys_green/254))
                
    blue_array = scaler.scale_array(blue_array, blue_band.GetNoDataValue(), blue_band.GetScale(), blue_band.GetOffset(),
                                    to_phys_blue, (to_phys_blue/254))
    
    
    # alpha
    # add alpha band to set no-data of RGB bands: this means we should not set the nodata values for the RGB bands
    alpha_array = np.where(
        np.logical_or(
            np.logical_or(red_array == output_nodata, green_array == output_nodata),
            blue_array == output_nodata), 0, 255).astype(np.dtype('B'))
    
    red_array = np.where(
        np.logical_or(green_array == output_nodata, blue_array == output_nodata),
        255,
        red_array).astype(np.dtype('B'))
    
    green_array = np.where(
        np.logical_or(red_array == output_nodata, blue_array == output_nodata),
        255, 
        green_array).astype(np.dtype('B'))
    
    blue_array = np.where(
        np.logical_or(red_array == output_nodata, green_array == output_nodata),
        255,
        blue_array).astype(np.dtype('B'))

    oldmask = os.umask(0o002)

    Helper.create_parent_dirs(output_file)

    driver = gdal.GetDriverByName("GTiff")
    output_raster = driver.Create(output_file, red_raster.RasterXSize, red_raster.RasterYSize,
                                  4, gdal.GDT_Byte, options=['COMPRESS=LZW', 'TILED=YES'])
    output_raster.SetGeoTransform(red_raster.GetGeoTransform())
    srs = osr.SpatialReference()
    srs.ImportFromEPSG(input_crs)
    output_raster.SetProjection(srs.ExportToWkt())

    # red
    red_output_band = output_raster.GetRasterBand(1)
    # red_output_band.SetNoDataValue(output_nodata)
    gdalnumeric.BandWriteArray(red_output_band, red_array)

    # green
    green_output_band = output_raster.GetRasterBand(2)
    # green_output_band.SetNoDataValue(output_nodata)
    gdalnumeric.BandWriteArray(green_output_band, green_array)

    # blue
    blue_output_band = output_raster.GetRasterBand(3)
    # blue_output_band.SetNoDataValue(output_nodata)
    gdalnumeric.BandWriteArray(blue_output_band, blue_array)

    
    red_array = None
    green_array = None
    blue_array = None
    alpha_output_band = output_raster.GetRasterBand(4)
    gdalnumeric.BandWriteArray(alpha_output_band, alpha_array)
    alpha_array = None

    output_raster.FlushCache()
    output_raster = None
    os.umask(oldmask)
    
    #create overviews
    os.system('gdaladdo ' + output_file + ' 2 4 8 16 32 64')


def read_band(input_path, band_name):

    input_file = Helper.build_netcdf_path(input_path, band_name)
    input_raster = gdal.Open(input_file, gdal.GA_ReadOnly)
    input_band = input_raster.GetRasterBand(1)
    return gdalnumeric.BandReadAsArray(input_band), input_band, input_raster


#def rescale(input_array, from_min, from_max, from_nodata, to_min, to_max, to_nodata):
#    
#    
#    input_array = np.where(
#        np.logical_and(input_array < from_min, input_array != from_nodata), from_min, input_array)
#    input_array = np.where(
#        np.logical_and(input_array > from_max, input_array != from_nodata), from_max, input_array)
#    return np.where(input_array == from_nodata,
#                    to_nodata,
#                    (to_max - to_min) / (from_max - from_min) * (input_array - from_nodata)).astype(np.dtype('B'))
