"""This DCS4COP post-processing module allows to process a DCS4COP NetCDF product"""
import os

from dcs4cop_vmm_postproc import scaler, masker, cloud2cube, merger_rgb
from dcs4cop_vmm_postproc.helper import Helper

#TODO add overviews to all products

def create_empty_file(infile):
    oldmask = os.umask(0o002)
    #writing .empy file
    Helper.create_parent_dirs(infile)
    output_file = infile.replace('.tiff','.empty')
    open(output_file, 'a').close()
    os.umask(oldmask)
    
def s2_tile(dcs4cop_product):

    if dcs4cop_product.hasNDAVI():
        #if NDAVI is available in the netcdf, also thecloudmaskband is updated with a newer version
        cloudmaskband = "CLOUD_OVER_LAND"
        cloudmaskband4cloudcover = cloudmaskband
    else:
        cloudmaskband = "FLAGS_SELECT"
    cloudmaskband_4_cloudcover = cloudmaskband  #used for cloudcover

    #quick and dirty hacks for DEME tiles, because deme workarea is falsly covered by clouds.
    if dcs4cop_product.tileID in ["36RTV","35RQQ"]:
        cloudmaskband = None

    """ RGB """
    merger_rgb.merge2rgb(dcs4cop_product.inFile,
                         #                      "toa_665", 0.03, 0.17,
                         #                      "toa_560", 0.03, 0.17,
                         #                      "toa_490", 0.03, 0.17,
                         "rrs_665", 0.00, 0.048,
                         "rrs_560", 0.00, 0.048,
                         "rrs_490", 0.00, 0.048,
                         255,
                         dcs4cop_product.outFileDict['rgb'])
    #only do RGB    
    """ CHL """
    outfile = masker.mask(dcs4cop_product.inFile, 
                         "CHL_MERGED",
                         "IDEPIX_LAND",
                         1,
                         '/data/DCS4COP1/postproc/input/watermask.tiff',
                         1,
                         cloudmaskband,
                         1,
                         dcs4cop_product.outFileDict['chl'])
    
    if outfile[-4:] == 'tiff' :
        #only scale if the file is not .empty
        scaler.scale(dcs4cop_product.outFileDict['chl'],
                     400,
                     1.6,
                     dcs4cop_product.outFileDict['chlScaled'])
    else:
        create_empty_file(dcs4cop_product.outFileDict['chlScaled'])

    
    """  TUR """
    outfile = masker.mask(dcs4cop_product.inFile, 
                          "TUR_NECHAD2009_GLOBAL_665",
                          "IDEPIX_LAND",
                          1,
                          '/data/DCS4COP1/postproc/input/watermask.tiff',
                          1,
                          cloudmaskband,
                          1,
                          dcs4cop_product.outFileDict['tur'])
    
    if outfile[-4:] == 'tiff' :
        scaler.scale(dcs4cop_product.outFileDict['tur'],
                     400,
                     1.6,
                     dcs4cop_product.outFileDict['turScaled'])
    else:
        create_empty_file(dcs4cop_product.outFileDict['turScaled'])
        
    """  SPM """
    outfile = masker.mask(dcs4cop_product.inFile, 
                          "SPM_NECHAD2010_GLOBAL_665",
                          "IDEPIX_LAND",
                          1,
                          '/data/DCS4COP1/postproc/input/watermask.tiff',
                          1,
                          cloudmaskband,
                          1,
                          dcs4cop_product.outFileDict['spm'])
    
    if outfile[-4:] == 'tiff' :
        scaler.scale(dcs4cop_product.outFileDict['spm'],
                     400,
                     1.6,
                     dcs4cop_product.outFileDict['spmScaled'])
    else:
        create_empty_file(dcs4cop_product.outFileDict['spmScaled'])

    if dcs4cop_product.hasNDAVI():
        '''NDAVI'''
        outfile = masker.mask(dcs4cop_product.inFile,
                              "NDAVI",
                              "IDEPIX_LAND",
                              1,
                              '/data/DCS4COP1/postproc/input/watermask.tiff',
                              1,
                              cloudmaskband,
                              1,
                              dcs4cop_product.outFileDict['ndavi'])

        if outfile[-4:] == 'tiff':
            scaler.scale(dcs4cop_product.outFileDict['ndavi'],
                         1,
                         0.008,
                         dcs4cop_product.outFileDict['ndaviScaled'],
                         min_phys_value=-1.0,
                         output_offset=-1.0
                         )
        else:
            create_empty_file(dcs4cop_product.outFileDict['ndaviScaled'])
        
    """" CLOUDMASK """
    cloud2cube.cloud2cube(dcs4cop_product.inFile,
                          cloudmaskband_4_cloudcover,
                           'IDEPIX_LAND',
                          1,
                           '/data/DCS4COP1/postproc/input/watermask.tiff',
                          1,
                          dcs4cop_product.outFileDict['cloudmask'])

    
def l8_scene(dcs4cop_product):
    """ RGB """
    print (dcs4cop_product.outFileDict['rgb'])
    merger_rgb.merge2rgb(dcs4cop_product.inFile,
                          "rrs_655", 0.02, 0.23,
                          "rrs_561", 0.02, 0.23,
                          "rrs_482", 0.02, 0.23,
                         255,
                         dcs4cop_product.outFileDict['rgb'])


    """  TUR """
    masker.mask(dcs4cop_product.inFile,
                "TUR_NECHAD2009_GLOBAL_655",
                cloudmaskband,
                1,
                "IDEPIX_CLOUD_SURE",
                1,
                dcs4cop_product.outFileDict['tur'])

    scaler.scale(dcs4cop_product.outFileDict['tur'],
                 100,
                 0.4,
                 dcs4cop_product.outFileDict['turScaled'])

    """  SPM """
    masker.mask(dcs4cop_product.inFile,
                "SPM_NECHAD2010_GLOBAL_655",
                "IDEPIX_LAND",
                1,
                "IDEPIX_CLOUD_SURE",
                1,
                dcs4cop_product.outFileDict['spm'])

    scaler.scale(dcs4cop_product.outFileDict['spm'],
                 100,
                 0.4,
                 dcs4cop_product.outFileDict['spmScaled'])

    """" CLOUDMASK """
    masker.mask(dcs4cop_product.inFile,
                "IDEPIX_CLOUD_SURE",
                "IDEPIX_LAND",
                1,
                "IDEPIX_CLOUD_SURE",
                1,
                dcs4cop_product.outFileDict['cloudmask'])
    # cloud2cube.cloud2cube(dcs4cop_product.inFile,
    #                       "IDEPIX_CLOUD_SURE",
    #                       "IDEPIX_LAND",
    #                       1,
    #                       dcs4cop_product.outFileDict['cloudmask'])
