"""This DCS4COP post-processing module allows to process a DCS4COP NetCDF product"""
import os

from dcs4cop_vmm_postproc import scaler, masker, cloud2cube, merger_rgb
from dcs4cop_vmm_postproc.helper import Helper

#TODO add overviews to all products

def create_empty_file(infile):
    oldmask = os.umask(0o002)
    #writing .empy file
    Helper.create_parent_dirs(infile)
    output_file = infile.replace('.tiff','.empty')
    open(output_file, 'a').close()
    os.umask(oldmask)
    
def s2_tile(dcs4cop_product):

    if dcs4cop_product.hasNDAVI():
        #if NDAVI is available in the netcdf, also thecloudmaskband is updated with a newer version
        cloudmaskband = "CLOUD_OVER_LAND"
    else:
        cloudmaskband = "FLAGS_SELECT"


    """ RGB """
    merger_rgb.merge2rgb(dcs4cop_product.inFile,
                         "rrs_665", 0.00, 0.048,
                         "rrs_560", 0.00, 0.048,
                         "rrs_490", 0.00, 0.048,
                         255,
                         dcs4cop_product.outFileDict['rgb'])
    #only do RGB    
    """ CHL """
    outfile = masker.mask(dcs4cop_product.inFile, 
                          "CHL_GILERSON2010_GLOBAL",
                          None,
                          1,
                          '/data/DCS4COP1/postproc/input/watermask.tiff',
                          1,
                          cloudmaskband,
                          1,
                          dcs4cop_product.outFileDict['chl'])
    
    if outfile[-4:] == 'tiff' :
        #only scale if the file is not .empty
        scaler.scale(dcs4cop_product.outFileDict['chl'],
                     400,
                     1.6,
                     dcs4cop_product.outFileDict['chlScaled'])
    else:
        create_empty_file(dcs4cop_product.outFileDict['chlScaled'])

    


    if dcs4cop_product.hasNDAVI():
        '''NDAVI'''
        outfile = masker.mask(dcs4cop_product.inFile,
                              "NDAVI",
                              None,
                              1,
                              '/data/DCS4COP1/postproc/input/watermask.tiff',
                              1,
                              cloudmaskband,
                              1,
                              dcs4cop_product.outFileDict['ndavi'])

        if outfile[-4:] == 'tiff':
            scaler.scale(dcs4cop_product.outFileDict['ndavi'],
                         1,
                         0.008,
                         dcs4cop_product.outFileDict['ndaviScaled'],
                         min_phys_value=-1.0,
                         output_offset=-1.0
                         )
        else:
            create_empty_file(dcs4cop_product.outFileDict['ndaviScaled'])
        
    """" CLOUDMASK """
    cloud2cube.cloud2cube(dcs4cop_product.inFile,
                          cloudmaskband,
                          None,
                          1,
                           '/data/DCS4COP1/postproc/input/watermask.tiff',
                          1,
                          dcs4cop_product.outFileDict['cloudmask'])
